from ohm2_handlers_light import utils as h_utils
from ohm2_accounts_light import utils as ohm2_accounts_light_utils

def user_information(request, user, previous_outputs, *args, **kwargs):
	output = {}
	
	user_information = kwargs.get("ohm2_user_information", {})
	if user_information:
		
		first_name, last_name = user_information["first_name"], user_information["last_name"]
		
		if first_name:
			user.first_name = first_name
		
		if last_name:
			user.last_name = last_name		
		

		user.save()

	return (user, output)



from django.utils.translation import ugettext as _

BASE_ERROR_CODE = 245696

NO_PASSWORD_RESET_FOUND = {
	"code" : BASE_ERROR_CODE | 1,
	"message" : _("No password reset found"),
}
THE_PASSWORD_IS_NOT_SECURE = {
	"code" : BASE_ERROR_CODE | 2,
	"message" : _("The password is not secure"),
}
EMAIL_IS_NOT_UNIQUE = {
	"code" : BASE_ERROR_CODE | 3,
	"message" : _("The email is not unique"),
}


# Generated by Django 2.0.1 on 2018-11-19 21:20

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone


class Migration(migrations.Migration):

    replaces = [('ohm2_accounts_light', '0001_initial'), ('ohm2_accounts_light', '0002_auto_20181119_2052'), ('ohm2_accounts_light', '0003_auto_20181119_2119')]

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='PasswordReset',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('identity', models.CharField(max_length=255, unique=True)),
                ('created', models.DateTimeField(default=django.utils.timezone.now)),
                ('last_update', models.DateTimeField(default=django.utils.timezone.now)),
                ('last_sent_date', models.DateTimeField(blank=True, default=None, null=True)),
                ('activation_date', models.DateTimeField(blank=True, default=None, null=True)),
                ('ip_address', models.GenericIPAddressField(blank=True, default='', null=True)),
                ('code', models.CharField(max_length=255, unique=True)),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



from django.utils.translation import ugettext as _

BASE_ERROR_CODE = 499264

USER_ALREADY_LOGGED_IN = {
	"code" : BASE_ERROR_CODE | 1,
	"message" : _("The user is already logged in"),
}
USER_ALREADY_EXIST = {
	"code" : BASE_ERROR_CODE | 2,
	"message" : _("The user already exist"),
}
THE_PASSWORD_IS_NOT_SECURE = {
	"code" : BASE_ERROR_CODE | 3,
	"message" : _("The password is not secure"),
}
INVALID_EMAIL = {
	"code" : BASE_ERROR_CODE | 4,
	"message" : _("The email is not valid"),
}
SIGNUPS_DISABLED = {
	"code" : BASE_ERROR_CODE | 5,
	"message" : _("Signups are currently disabled"),
}
EMAIL_LOGIN_DISABLED = {
	"code" : BASE_ERROR_CODE | 6,
	"message" : _("Email login is disabled"),
}
WRONG_CREDENTIALS = {
	"code" : BASE_ERROR_CODE | 7,
	"message" : _("Wrong credentials"),
}
PASSWORD_RESET_IDENTITY_AND_CODE_CANT_BE_BOTH_EMPTY = {
	"code" : BASE_ERROR_CODE | 8,
	"message" : _("Identity and code can't be both empty"),
}
PASSWORD_RESET_INVALID = {
	"code" : BASE_ERROR_CODE | 9,
	"message" : _("Password reset invalid"),
}
PASSWORD_RESET_ALREADY_ACTIVATED = {
	"code" : BASE_ERROR_CODE | 10,
	"message" : _("Password reset already activated"),
}
SIGNUP_PIPELINE_FAILED = {
	"code" : BASE_ERROR_CODE | 11,
	"message" : _("Signup pipeline failed"),
}
SIGNUP_FAILED = {
	"code" : BASE_ERROR_CODE | 12,
	"message" : _("Signup failed"),
}
INVALID_FACEBOOK_ACCESS_TOKEN = {
	"code" : BASE_ERROR_CODE | 13,
	"message" : _("Invalid facebook access token"),
}
FACEBOOK_CONNECTION_ERROR = {
	"code" : BASE_ERROR_CODE | 14,
	"message" : _("Facebook connection error"),
}
FACEBOOK_EMAIL_PERMISSION_NOT_SETTED = {
	"code" : BASE_ERROR_CODE | 15,
	"message" : _("Facebook email permission not setted"),
}
INVALID_GOOGLE_PLUS_ACCESS_TOKEN = {
	"code" : BASE_ERROR_CODE | 16,
	"message" : _("Invalid google plus access token"),
}
GOOGLE_PLUS_CONNECTION_ERROR = {
	"code" : BASE_ERROR_CODE | 17,
	"message" : _("Google Plus connection error"),
}
GOOGLE_PLUS_NO_EMAILS_ASSOCIATED_WITH_THE_ACCOUNT = {
	"code" : BASE_ERROR_CODE | 18,
	"message" : _("Google Plus no emails associated with the account"),
}
SIGNUP_FAILED_DUE_TO_TOKEN = {
	"code" : BASE_ERROR_CODE | 19,
	"message" : _("Signup failed due to token"),
}
LOGIN_FAILED = {
	"code" : BASE_ERROR_CODE | 20,
	"message" : _("Login failed"),
}
LOGIN_FAILED_DUE_TO_TOKEN = {
	"code" : BASE_ERROR_CODE | 21,
	"message" : _("Login failed due to token"),
}
FACEBOOK_ERROR_VALIDATING_ACCESS_TOKEN = {
	"code" : BASE_ERROR_CODE | 22,
	"message": _("The user has not authorized application"),
}


from django.contrib.auth.models import User as AuthUser
from rest_framework import serializers
from ohm2_accounts_light import models as ohm2_accounts_light_models



class User(serializers.ModelSerializer):
	class Meta:
		model = AuthUser
		fields = (
			'username',
			'email',
			'last_name',
			'first_name',
		)




from ohm2_handlers_light import utils as h_utils
from ohm2_accounts_light.decorators import ohm2_accounts_light_safe_request
from ohm2_accounts_light import utils as ohm2_accounts_light_utils
from ohm2_accounts_light import settings
from . import errors as api_v1_errors
from . import settings as api_v1_settings
import requests

if api_v1_settings.FACEBOOK_LOGIN:
	import facebook



@ohm2_accounts_light_safe_request
def signup(request, params, **pipeline_options):
	p = h_utils.cleaned(params, (
							("string", "username", 1),
							("string", "password", 1),
							("string", "email", 0),
							("def_dict", "user_information", (
								("first_name", ""),
								("last_name", ""),
							)),
						))

	if request.user.is_authenticated:
		return {"error": api_v1_errors.USER_ALREADY_LOGGED_IN}
	
	elif ohm2_accounts_light_utils.user_exist(username = p["username"]):
		return {"error" : api_v1_errors.USER_ALREADY_EXIST}

	elif settings.CHECK_PASSWORD_SECURE and not ohm2_accounts_light_utils.is_password_secure(p["password"]):
		return {"error" : api_v1_errors.THE_PASSWORD_IS_NOT_SECURE}

	elif len(p["email"]) > 0 and not h_utils.is_email_valid(p["email"]):
		return {"error" : api_v1_errors.INVALID_EMAIL}

	elif not settings.SIGNUPS_ENABLED:
		return {"error" : api_v1_errors.SIGNUPS_DISABLED}

	else:
		pass


	username, password, email = p["username"], p["password"], p["email"]
	if len(email) == 0 and h_utils.is_email_valid(username):
		email = username

	

	pipeline_options["username"] = username
	pipeline_options["email"] = email
	pipeline_options["password"] = password

	
	pipeline_options["ohm2_user_information"] = p["user_information"]


	user = ohm2_accounts_light_utils.create_user(username, email, password)
	try:
		ohm2_accounts_light_utils.run_signup_pipeline(request, user, **pipeline_options)
	except Exception as e:
		ohm2_accounts_light_utils.delete_user(user)
		return {"error" : api_v1_errors.SIGNUP_PIPELINE_FAILED}
	

	res = {
		"error" : None,
		"ret" : True,
	}
	return res



@ohm2_accounts_light_safe_request
def login(request, params):
	p = h_utils.cleaned(params, (
							("string", "username", 1),
							("string", "password", 1),
						))


	if request.user.is_authenticated:
		return {"error": None, "ret" : True}
	
	else:
		username, password = p["username"], p["password"]

		
	if not settings.ENABLE_EMAIL_LOGIN and h_utils.is_email_valid(username):
		return {"error" : api_v1_errors.EMAIL_LOGIN_DISABLED}

	elif settings.ENABLE_EMAIL_LOGIN and h_utils.is_email_valid(username) and ohm2_accounts_light_utils.user_exist(email = username) and settings.UNIQUE_USER_EMAILS:
		user = ohm2_accounts_light_utils.get_user(email = username)
		auth_user = ohm2_accounts_light_utils.user_authenticate(user.get_username(), password)
	
	else:
		auth_user = ohm2_accounts_light_utils.user_authenticate(username, password)
	
	
	if auth_user is None:
		return {"error" : api_v1_errors.WRONG_CREDENTIALS}

	else:
		ohm2_accounts_light_utils.run_login_pipeline(request, auth_user)	
	

	res = {
		"error" : None,
		"ret" : True,
	}
	return res



@ohm2_accounts_light_safe_request
def logout(request, params):
	p = h_utils.cleaned(params,	(
						))


	if not request.user.is_authenticated:
		return {"error": None, "ret" : False}
	
	else:
		ohm2_accounts_light_utils.run_logout_pipeline(request)	
	

	res = {
		"error" : None,
		"ret" : True,
	}
	return res


@ohm2_accounts_light_safe_request
def signup_and_get_token(request, params):
	p = h_utils.cleaned(params, (
							("string", "username", 1),
						))


	username = p["username"]

	res, error = signup(request, params)
	if error:
		return {"error" : error.regroup()}
	
	elif res.get("error"):
		return {"error" : res.get("error")}

	elif not res.get("ret", False):
		return {"error" : api_v1_errors.SIGNUP_FAILED}	

	else:
		user = ohm2_accounts_light_utils.get_user(username = username)

	try:
		token = ohm2_accounts_light_utils.get_or_create_authtoken(user)
	except Exception as e:
		return {"error" : api_v1_errors.SIGNUP_FAILED_DUE_TO_TOKEN}
	
	
	res = {
		"error" : None,
		"ret" : {
			"token" : token.key,
		}
	}
	return res



@ohm2_accounts_light_safe_request
def login_and_get_token(request, params):
	p = h_utils.cleaned(params, (
							("string", "username", 1),
						))


	username = p["username"]

	res, error = login(request, params)
	if error:
		return {"error" : error.regroup()}
	
	elif res.get("error"):
		return {"error" : res.get("error")}

	elif not res.get("ret", False):
		return {"error" : api_v1_errors.LOGIN_FAILED}	

	else:
		user = ohm2_accounts_light_utils.get_user(username = username)

	try:
		token = ohm2_accounts_light_utils.get_or_create_authtoken(user)
	except Exception as e:
		return {"error" : api_v1_errors.LOGIN_FAILED_DUE_TO_TOKEN}
	
	
	res = {
		"error" : None,
		"ret" : {
			"token" : token.key,
		}
	}
	return res



@ohm2_accounts_light_safe_request
def create_authtoken(request, params):
	p = h_utils.cleaned(params,	(
							("string", "username", 1),
						))


	user = ohm2_accounts_light_utils.get_user(username = p["username"])
	token = ohm2_accounts_light_utils.get_or_create_authtoken(user)

	res = {
		"error" : None,
		"ret" : {
			"token" : token.key,
		}
	}
	return res



@ohm2_accounts_light_safe_request
def get_authtoken(request, params):
	p = h_utils.cleaned(params,	(
							("string", "username", 1),
						))


	user = ohm2_accounts_light_utils.get_user(username = p["username"])
	token = ohm2_accounts_light_utils.get_or_create_authtoken(user)

	res = {
		"error" : None,
		"ret" : {
			"token" : token.key,
		}
	}
	return res



@ohm2_accounts_light_safe_request
def send_password_reset_link(request, params):
	p = h_utils.cleaned(params,	(
							("string", "username", 1),
						))


	if request.user.is_authenticated:
		return {"error": None, "ret" : False}

	username = p["username"]

	if h_utils.is_email_valid(username) and settings.UNIQUE_USER_EMAILS:
		user = ohm2_accounts_light_utils.get_or_none_user(email = username)

	else:	
		user = ohm2_accounts_light_utils.get_or_none_user(username = username)
	
	if user is None:
		return {"error": None, "ret" : False}
		
	passwordreset = ohm2_accounts_light_utils.get_or_none_passwordreset(user = user, activation_date = None)
	if passwordreset is None:
		passwordreset = ohm2_accounts_light_utils.create_passwordreset(user)

	
	sent = False
	if passwordreset.send_again() and settings.PASSWORD_RESET_SEND_ENABLE:

		passwordreset, sent = ohm2_accounts_light_utils.send_passwordreset(passwordreset, request)
		


	res = {
		"error" : None,
		"ret" : sent,
	}
	return res



@ohm2_accounts_light_safe_request
def set_password_reset(request, params):
	p = h_utils.cleaned(params,	(
							("string", "username", 1),
							("string", "password", 1),
							("string", "identity", 0),
							("string", "code", 0),
						))


	if request.user.is_authenticated:
		return {"error": None, "ret" : False}

	username, identity, code, password = p["username"], p["identity"], p["code"], p["password"]

	lookup = {}

	if len(identity) > 0:
		lookup["identity"] = identity

	elif len(code) > 0:
		lookup["code__iexact"] = code
	
	else:
		return {"error" : api_v1_errors.PASSWORD_RESET_IDENTITY_AND_CODE_CANT_BE_BOTH_EMPTY}


	if h_utils.is_email_valid(username) and settings.UNIQUE_USER_EMAILS:
		lookup["user__email"] = username

	else:
		lookup["user__username"] = username
	

	passwordreset = ohm2_accounts_light_utils.get_or_none_passwordreset(**lookup)
	if passwordreset is None:
		return {"error" : api_v1_errors.PASSWORD_RESET_INVALID}
	
	elif passwordreset.last_sent_date == None:
		return {"error" : api_v1_errors.PASSWORD_RESET_INVALID}

	elif passwordreset.activation_date != None:
		return {"error" : api_v1_errors.PASSWORD_RESET_ALREADY_ACTIVATED}

	elif settings.CHECK_PASSWORD_SECURE and not ohm2_accounts_light_utils.is_password_secure(password):
		return {"error" : api_v1_errors.THE_PASSWORD_IS_NOT_SECURE}

	else:
		passwordreset = ohm2_accounts_light_utils.change_password_with_passwordreset(passwordreset, password)

		
	res = {
		"error" : None,
		"ret" : True,
	}
	return res



@ohm2_accounts_light_safe_request
def update_user_information(request, params):
	p = h_utils.cleaned(params, (						
							("string", "first_name", 0),
							("string", "last_name", 0),
							("string", "email", 0),
						))

	
	user = ohm2_accounts_light_utils.get_user(username = request.user.get_username())
	
	user = h_utils.db_update(user, first_name = p["first_name"], last_name = p["last_name"], email = p["email"])
	
	ret = {
		"error" : None,
		"ret" : True,
	}
	return ret



def process_social_login_and_get_token(request, username, email, **options):
	user = ohm2_accounts_light_utils.get_or_none_user(username = username)
	if user is None:
		user_created = True

		password = h_utils.random_string(32)
		params = {
			"username": username,
			"email": email,
			"password": password,
			"user_information": {},
		}
		res, error = signup(request, params, **options)
		if error:
			return {"error" : error.regroup()}
		
		elif res.get("error"):
			return {"error" : res.get("error")}

		elif not res.get("ret", False):
			return {"error" : api_v1_errors.SIGNUP_FAILED}	

		user = ohm2_accounts_light_utils.get_user(username = username)
	else:
		user_created = False
	
	token = ohm2_accounts_light_utils.get_or_create_authtoken(user)	
	res = {
		"error" : None,
		"ret" : {
			"username": username,
			"token" : token.key,
			"user_created": user_created,
		}
	}
	return res




@ohm2_accounts_light_safe_request
def facebook_login_and_get_token(request, params):
	p = h_utils.cleaned(params, (
							("string", "access_token", 10),
						))

	try:
		graph = facebook.GraphAPI(access_token = p["access_token"])
	except Exception as e:
		return {"error" : api_v1_errors.INVALID_FACEBOOK_ACCESS_TOKEN}

		
	try:
		me = graph.get_object(id = "me?fields=" + settings.FACEBOOK_ME_FIELDS)
	except Exception as e:
		if hasattr(e, "code"):
			if e.code == 190:
				return {"error" : api_v1_errors.FACEBOOK_ERROR_VALIDATING_ACCESS_TOKEN}
		return {"error" : api_v1_errors.FACEBOOK_CONNECTION_ERROR}

	email = me.get("email")
	if email is None:
		return {"error" : api_v1_errors.FACEBOOK_EMAIL_PERMISSION_NOT_SETTED}
	
	username = email
	
	options = {
		"facebook": {
			"response": me,
			"kwargs": {},
		}
	}
	return process_social_login_and_get_token(request, username, email, **options)




@ohm2_accounts_light_safe_request
def google_plus_login_and_get_token(request, params):
	p = h_utils.cleaned(params, (
							("string", "access_token", 10),
						))

	
	try:
		r = requests.get(settings.GOOGLE_PLUS_PEOPLE_ME_URL, params = {'access_token': p["access_token"]})
	except Exception as e:
		return {"error" : api_v1_errors.GOOGLE_PLUS_CONNECTION_ERROR}

	
	data = r.json()
	if data.get("error"):
		return {"error" : api_v1_errors.INVALID_GOOGLE_PLUS_ACCESS_TOKEN}
	
	emails = data.get("emails", [])
	if len(emails) == 0:
		return {"error" : api_v1_errors.GOOGLE_PLUS_NO_EMAILS_ASSOCIATED_WITH_THE_ACCOUNT}
	
	username = email = emails[0]["value"]

	options = {}

	"""
	OH: Get image
	"""

	return process_social_login_and_get_token(request, username, email, **options)








import os
from setuptools import setup

README = open(os.path.join(os.path.dirname(__file__), 'README.rst')).read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name = 'django-ohm2-accounts-light',
    version = '0.5.4',
    packages = ['ohm2_accounts_light'],
    install_requires = [
        'django-ohm2-handlers-light',
    ],
    include_package_data = True,
    license = 'BSD License',
    description = 'Django application to easily handle signup, login and logout',
    long_description = 'README',
    url = 'http://www.ohm2.cl/',
    author = 'Oliver Herrera',
    author_email = 'oliver@ohm2.cl',
    classifiers = [
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License', # example license
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
)
